;|
MG.LSP (Make Group)

Programme zur interaktiven, einfachen Anwendung von AutoCAD-Gruppen.

Hintergrund:
Der Nutzen von AutoCAD-Gruppen wird oft unterschtzt.
Gerade im Bereich der temporren Verwendung
("Ach, dann fasse ich mal eben die Objekte zu einer Gruppe
zusammen, die brauche ich nachher doch nochmal...")
lassen sich Gruppen optimal einsetzen.
Leider ist der Gruppenbefehl in AutoCAD sowohl in der Kommandozeile
als auch im Dialogfenster "Gruppe" fr diesen Zweck viel zu umstndlich.
Und genau hier setzen diese Tools in Verbindung mit dem dazugehrigen
Werkzeugkasten an...

Die Befehle:

"MG"   -> (Make Group)
          erzeugt Gruppen mittels Objektwahl (auch implizite Wahl mglich).
          Der Gruppenname wird automatisch als "Timestamp"
          (TT-MMMM-JJJJ-HH-MM-SS) vergeben.

"ADDG" -> (Add Group-elements)
          Fgt zu whlende Objekte einer zu whlenden Gruppe hinzu.

"DELG" -> (Delete Group-elements)
          Entfernt zu whlende Objekte aus einer zu whlenden Gruppe.

"XG"   -> (eXplode Group)
          Lst eine zu whlende Gruppe auf.

"XGA"  -> (eXplode Group All)
          Lst ALLE Gruppen einer Zeichnung auf.

"TPS"  -> (Toggle PickStyle)
          schaltet die Mglichkeit zur Gruppenwahl
          ein oder aus.


Autor: Marc Scherer
       marc.scherer@zvo.com
       erstellt am 17. Juli 2003
       fr die HERDT-Schulungsunterlage
       AutoCAD 2004 - 2D-Aufbaukurs (Autor: Peter Tropf)
|;

;;; Erstellt eine Gruppe anhand auszuwhlender Objekte
(defun C:MG (/ GNAME SGET)
  (TOGGLE-GROUPSELECT t) ;_ Sicherstellen, dass Gruppenwahl mglich ist
  (princ "\nZu gruppierende Objekte whlen:")
  (setq SGET  (ssget)
        GNAME (menucmd
                "M=$(edtime,$(getvar,date),DD-MONTH-YYYY-HH-MM-SS)" ;_ erzeuge Timestamp-Gruppen-Namen
              ) ;_ end menucmd
  ) ;_ end setq
  (if SGET
    (progn
      (command "_.-group"          "_c"
               GNAME               "Automatisch erstellte Gruppe"
               SGET                ""
      ) ;_ end of command
      (sssetfirst NIL SGET)
      (princ "Gruppe erstellt. AutoCAD-Befehl eingeben...")
    ) ;_ end progn
    (princ "\nKeine Objekte gewhlt...")
  ) ;_ end if
  (princ)
) ;_ end defun


;;; Lst die Gruppe(n) auf, zu der/denen das zu klickende Objekt gehrt
(defun C:XG (/ GNAME GNAMELEN SGET)
  (if (not (GET-GNAMESFROMDIC)) ;_ Gruppen berhaupt vorhanden?
    (princ "\nKEINE Gruppen vorhanden!")
    (progn
      (setq SGET
	     (entsel
	       "\nObjekt whlen, dessen Gruppe(n) aufgelst werden soll(en): "
	     ) ;_ end entsel
      ) ;_ end of setq
      (if SGET
	(progn
	  (setq	SGET  (entget (car SGET))
		GNAME (GET-GROUPNAME SGET)
	  ) ;_ end of setq
	  (if GNAME
	    (progn
	      (setq GNAMELEN (itoa (length GNAME)))
	      (foreach ELEM GNAME
		(command "_.-group" "_ex" ELEM)
	      ) ;_ end foreach
	      (princ
		(strcat "\nOk, \"" GNAMELEN "\" Gruppe(n) aufgelst!")
	      ) ;_ end of princ
	    ) ;_ end progn
	    (princ "\nGewhltes Objekt gehrt zu KEINER Gruppe!")
	  ) ;_ end if
	) ;_ end progn
	(princ "\nKein Objekt gewhlt...!")
      ) ;_ end if
    ) ;_ end of progn
  ) ;_ end of if
  (princ)
) ;_ end defun

;;; Lst alle Gruppen auf
(defun C:XGA (/ GLIST GLIST2)
  (setq GLIST  (GET-GNAMESFROMDIC)
        GLIST2 '()
  ) ;_ end setq
  (if GLIST
    (progn
      (foreach ELEM GLIST
        (setq GLIST2 (cons (cdr ELEM) GLIST2))
      ) ;_ end foreach
      (foreach ELEM GLIST2
        (command "_.-group" "_ex" ELEM)
      ) ;_ end foreach
      (princ (strcat "\nOk, \""
                     (itoa (length GLIST2))
                     "\" Gruppe(n) wurden aufgelst!"
             ) ;_ end strcat
      ) ;_ end princ
    ) ;_ end progn
    (princ "\nKEINE Gruppen vorhanden!")
  ) ;_ end if
  (princ)
) ;_ end defun

;;; Funktion fgt Objekte zu einer vorhandenen Gruppe hinzu
(defun C:ADDG (/ GNAME GROUPSEL MRKSET SGET ALT-PICKSTYLE)
  (if (not (GET-GNAMESFROMDIC)) ;_ Gruppen berhaupt vorhanden?
    (princ "\nKEINE Gruppen vorhanden!")
    (progn
      (setq GROUPSEL	  (entsel
			    "\nGruppe whlen, der Objekte hinzugefgt werden sollen: "
			  ) ;_ end of entsel
	    ALT-PICKSTYLE (getvar "pickstyle")
      ) ;_ end of setq
      (if GROUPSEL
	(progn
	  (setq	GNAME (GET-GROUPNAME
			(entget
			  (car
			    GROUPSEL
			  ) ;_ end car
			) ;_ end of entget
		      ) ;_ end of get-groupname
	  ) ;_ end of setq
	  (if GNAME
	    (progn
	      (TOGGLE-GROUPSELECT t)
	      (command "_.select" "_group" (car GNAME) "")
	      (setq MRKSET (ssget "_P"))
	      (MARK-UNMARK MRKSET 3) ;_ Objekte der Gruppe highlighten (ausleuchten)
	      (princ
		"\nObjekte whlen, die der Gruppe hinzugfgt werden sollen..."
	      ) ;_ end of princ
	      (setq SGET (ssget))
	      (if SGET
		(progn
		  (command "_.-group" "_add" (car GNAME) SGET "")
		  (princ (strcat "\nOK, Objekte der Gruppe: \""
				 (car GNAME)
				 "\" hinzugefgt."
			 ) ;_ end of strcat
		  ) ;_ end of princ
		) ;_ end of progn
		(princ "\nKeine Objekte gewhlt, Funktionsende !")
	      ) ;_ end of if
	      (MARK-UNMARK MRKSET 4) ;_ Objekthighlighting der Gruppe entfernen
	      (setvar "pickstyle" ALT-PICKSTYLE)
	    ) ;_ end of progn
	    (princ "\nKeine Gruppe gewhlt, Funktionsende !")
	  ) ;_ end of if
	) ;_ end of progn
	(princ "\nNichts gewhlt, Funtionsende !")
      ) ;_ end of if
    ) ;_ end of progn
  ) ;_ end of if
  (princ)
) ;_ end of defun


;;; Funktion entfernt Objekte aus einer vorhandenen Gruppe
(defun C:DELG (/ GNAME GROUPSEL MRKSET SGET ALT-PICKSTYLE SGET2)
  (if (not (GET-GNAMESFROMDIC)) ;_ Gruppen berhaupt vorhanden?
    (princ "\nKEINE Gruppen vorhanden!")
    (progn
      (setq GROUPSEL	  (entsel
			    "\nGruppe whlen, aus der Objekte entfernt werden sollen: "
			  ) ;_ end of entsel
	    ALT-PICKSTYLE (getvar "pickstyle")
      ) ;_ end of setq
      (if GROUPSEL
	(progn
	  (setq	GNAME (GET-GROUPNAME
			(entget
			  (car
			    GROUPSEL
			  ) ;_ end car
			) ;_ end of entget
		      ) ;_ end of get-groupname
	  ) ;_ end of setq
	  (if GNAME
	    (progn
	      (TOGGLE-GROUPSELECT t)
	      (command "_.select" "_group" (car GNAME) "")
	      (setq MRKSET (ssget "_P"))
	      (MARK-UNMARK MRKSET 3) ;_ Objekte der Gruppe highlighten (ausleuchten)
	      (princ
		"\nObjekte whlen, die aus der Gruppe entfernt werden sollen..."
	      ) ;_ end of princ
	      (TOGGLE-GROUPSELECT NIL)
	      (setq SGET (ssadd))
	      (while (setq SGET2 (ssget "_:S"))
		(MARK-UNMARK SGET2 4)
		(SSUNION (list SGET SGET2))
	      ) ;_ end of while
	      (if (> (sslength SGET) 0)
		(progn
		  (command "_.-group" "EN" (car GNAME) SGET "")
		  (princ (strcat "\nOK, Objekte aus der Gruppe: \""
				 (car GNAME)
				 "\" entfernt."
			 ) ;_ end of strcat
		  ) ;_ end of princ
		) ;_ end of progn
		(princ "\nKeine Objekte gewhlt, Funktionsende !")
	      ) ;_ end of if
	      (MARK-UNMARK MRKSET 4) ;_ Objekthighlighting der Gruppe entfernen
	      (setvar "pickstyle" ALT-PICKSTYLE)
	    ) ;_ end of progn
	    (princ "\nKeine Gruppe gewhlt, Funktionsende !")
	  ) ;_ end of if
	) ;_ end of progn
	(princ "\nNichts gewhlt, Funtionsende !")
      ) ;_ end of if
    ) ;_ end of progn
  ) ;_ end of if
  (princ)
) ;_ end of defun


;;; Funktion schaltet Gruppenwahl ein oder aus.
;;; Wenn an, gehts aus und umgekehrt...
(defun C:TPS (/)
  (if (= (logand 1 (getvar "pickstyle")) 0)
    (princ (strcat "\nOK, Gruppenwahl AN. \"PICKSTYLE\" = "
                   (itoa (TOGGLE-GROUPSELECT t))
           ) ;_ end of strcat
    ) ;_ end of princ
    (princ (strcat "\nOK, Gruppenwahl AUS. \"PICKSTYLE\" = "
                   (itoa (TOGGLE-GROUPSELECT NIL))
           ) ;_ end of strcat
    ) ;_ end of princ
  ) ;_ end of if
  (princ)
) ;_ end of defun

;;; SubFunction: Fhrt zwei Selection-Sets zusammen.
;;; bergabe Liste aus Selection-Sets.
;;; In das erste SelectionSet der Liste gehen die anderen ein.
;;; Rckgabe: Lnge des resultierenden Auswahlsatzes
(defun SSUNION (SSLIST)
  (mapcar '(lambda (X / C)
             (setq C -1)
             (repeat (sslength X)
               (ssadd (ssname X (setq C (1+ C))) (car SSLIST))
             ) ;_ end of repeat
           ) ;_ end of lambda
          (cdr SSLIST)
  ) ;_ end of mapcar
  (sslength (car SSLIST))
) ;_ end of defun

;;; SubFunction: Markiert Objekte aus Auswahlsatz oder hebt Markierung auf.
;;; bergabe: Auswahlsatz und Modus: 3 = Markieren 4 = Markierung aufheben
(defun MARK-UNMARK (SELSET MODE / INDEX)
  (setq INDEX 0)
  (while (< INDEX (sslength SELSET))
    (redraw (ssname SELSET INDEX) MODE)
    (setq INDEX (1+ INDEX))
  ) ;_ end of while
) ;_ end of defun

;;; SubFunction: Gibt die Gruppennamen zurck, zu denen ein Objekt gehrt
(defun GET-GROUPNAME
       (DATA / 330DATA GLIST GNAME GNAMES INDEX OBJID RETVAL)
  (setq OBJID  (cdr (assoc -1 DATA)) ;_ ObjektID um rauszufinden, in welchen Gruppen das Objekt vorkommt
        RETVAL (assoc 102 DATA) ;_ Kommt nur vor, wenn ein ACAD_REACTOR existiert
        INDEX  0 ;_ init
        GLIST  '() ;_ init
  ) ;_ end setq
  (if RETVAL ;_ wenn GC 102 existiert
    (progn
      (while (< INDEX (length DATA))
        (if (= (car (nth INDEX DATA)) 330)
          (progn
            (setq 330DATA (entget (cdr (nth INDEX DATA))))
            (if (= (cdr (assoc 0 330DATA)) "GROUP") ;_ wenn das 'ne Gruppe ist
              (progn
                (if (not GNAMES)
                  (setq GNAMES (GET-GNAMESFROMDIC))
                ) ;_ end if
                (setq
                  GNAME (cdr (assoc (cdr (assoc -1 330DATA)) GNAMES))
                ) ;_ end setq
                (if (not (member GNAME GLIST))
                  (setq GLIST (cons GNAME GLIST))
                ) ;_ end if
              ) ;_ end progn
            ) ;_ end if
          ) ;_ end progn
        ) ;_ end if
        (setq INDEX (1+ INDEX))
      ) ;_ end while
      (setq RETVAL GLIST)
    ) ;_ end progn
  ) ;_ end if
  GLIST
) ;_ end defun


;;; SubFunction: Erstellt 'ne Liste mit allen Gruppennamen und deren Objekt-ID
(defun GET-GNAMESFROMDIC (/ CODE GROUPLIST GRPDICT INDEX OBJDICT)
  (setq OBJDICT   (namedobjdict)
        GRPDICT   (dictsearch OBJDICT "ACAD_GROUP")
        GROUPLIST '()
        INDEX     0
  ) ;_ end setq
  (while (< INDEX (length GRPDICT))
    (if (= (setq CODE (car (nth INDEX GRPDICT))) 3)
      (setq GROUPLIST (cons (cons (cdr (nth (1+ INDEX) GRPDICT))
                                  (cdr (nth INDEX GRPDICT))
                            ) ;_ end cons
                            GROUPLIST
                      ) ;_ end cons
      ) ;_ end setq        
    ) ;_ end if
    (setq INDEX (1+ INDEX))
  ) ;_ end while
  GROUPLIST
) ;_ end defun

;;; SubFunction: Verndert den Wert der Sysvar "Pickstyle".
;;; Gruppenwahl ist aktiv, wenn das Bit 1 gesetzt ist.
;;; Funktion ist als Switch ausgelegt, dient also sowohl dem
;;; Aktivieren als auch dem Deaktivieren.
;;; Gruppenwahl wird aktiviert, wenn ein Parameter ungleich nil
;;; bergeben wird, und deaktiviert, wenn als Parameter nil bergeben wird.
;;; -> Schalterfunktion: T = an, nil = aus
;;; <- Status von Pickstyle
(defun TOGGLE-GROUPSELECT (FLAG / IS-INACTIVE?)
  (setq IS-INACTIVE? (= (logand 1 (getvar "pickstyle")) 0))
  (if FLAG
    (if IS-INACTIVE?
      (setvar "pickstyle" (1+ (getvar "pickstyle")))
    ) ;_ end of if
    (if (not IS-INACTIVE?)
      (setvar "pickstyle" (1- (getvar "pickstyle")))
    ) ;_ end of if
  ) ;_ end of if
  (getvar "pickstyle")
) ;_ end of defun


(defun C:GTOLAY (/ GNAME)
  (if (setq GNAME (GLST->STR))
    (progn
      (command "_.-layer" "_n" GNAME "")
      (princ "\nOkay, Layer erstellt...")
    ) ;_ end of progn
    (alert "Keine Gruppen in Zeichnung vorhanden...")
  ) ;_ end of if
  (princ)
) ;_ end of defun


;;; erzeugt aus einer Gruppenliste von "get-gnamesfromDic"
;;; einen Comma-delimited String zum Erzeugen der Layernamen=Gruppennamen
(defun GLST->STR (/ LST STR X)
  (setq LST (GET-GNAMESFROMDIC)) ;_ end of setq
  (if LST
    (setq LST (mapcar '(lambda (X) (strcat "," (cdr X))) LST)
          STR (apply 'strcat LST)
    ) ;_ end of setq
  ) ;_ end of if
  STR
) ;_ end of defun

(princ
  "\nFunktionen fr AutoCAD-Gruppen: \"MG\" erstellt, \"XG\" lst auf, \"XGA\" lst ALLE auf,"
) ;_ end of princ
(princ
  "\n\"ADDG\"=Objekte zur Gruppe hinzu, \"DELG\"=Objekte aus Gruppe entfernen, \"TPS\"=Gruppenwahl Ein/Aus..."
) ;_ end of princ
(princ)
